﻿using System;
using System.Collections.Generic;
using System.Net;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.Text;
using SecureClientApp.SecureService1;
using SecureClientApp.SecureService2;
using SecureClientApp.SecureService3;
using SecureClientApp.SecureService4;
using SecureClientApp.SecureService5;

namespace SecureClientApp
{
	class ClientApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Simple Secure Client Sample......");

			InvokeService1();
			InvokeService2();
			InvokeService3();
			InvokeService4();
			InvokeService5();
		}

		// 보안을 사용하지 않는 서비스 호출
		private static void InvokeService1()
		{
			// ChannelFactory<T>를 사용한 예제 코드
			BasicHttpBinding binding = new BasicHttpBinding();
			binding.Security.Mode = BasicHttpSecurityMode.None;

			ServiceEndpoint ep = new ServiceEndpoint(
				ContractDescription.GetContract(typeof(ISecureService1)),
				binding,
				new EndpointAddress("http://localhost/wcf/example/security/test/service1"));

			ChannelFactory<ISecureService1> factory =
				new ChannelFactory<ISecureService1>(ep);
			ISecureService1 proxy1 = factory.CreateChannel();
			using (proxy1 as IDisposable) {
				proxy1.SecureMethod1("Non-secure service call");
				Console.WriteLine("SecureService1 (non-secure) invoked...");
			}
			// ClientBase<T>를 사용한 예제 코드
			using (SecureService1Client proxy2 = new SecureService1Client()) {
				proxy2.SecureMethod1("Non-secure service call");
				Console.WriteLine("SecureService1 (non-secure) invoked...");
			}
		}

		// SSL과 익명 인증을 사용하는 서비스 호출
		private static void InvokeService2()
		{
			// ChannelFactory<T>를 사용한 예제 코드
			BasicHttpBinding binding = new BasicHttpBinding();
			binding.Security.Mode = BasicHttpSecurityMode.Transport;
			binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.None;

			ServiceEndpoint ep = new ServiceEndpoint(
				ContractDescription.GetContract(typeof(ISecureService2)),
				binding,
				new EndpointAddress("https://localhost/wcf/example/security/test/service2"));

			ChannelFactory<ISecureService2> factory =
				new ChannelFactory<ISecureService2>(ep);
			ISecureService2 proxy1 = factory.CreateChannel();
			using (proxy1 as IDisposable) {
				proxy1.SecureMethod2("SSL with anonymous authentication service call");
				Console.WriteLine("SecureService2 (using SSL with anonymous authentication) invoked...");
			}
			// ClientBase<T>를 사용한 예제 코드
			using (SecureService2Client proxy2 = new SecureService2Client()) {
				proxy2.SecureMethod2("SSL with anonymous authentication service call");
				Console.WriteLine("SecureService2 (using SSL with anonymous authentication) invoked...");
			}
		}

		// SSL과 Windows 인증을 사용하는 서비스 호출
		private static void InvokeService3()
		{
			// ChannelFactory<T>를 사용한 예제 코드
			BasicHttpBinding binding = new BasicHttpBinding();
			binding.Security.Mode = BasicHttpSecurityMode.Transport;
			binding.Security.Transport.ClientCredentialType = HttpClientCredentialType.Windows;

			ServiceEndpoint ep = new ServiceEndpoint(
				ContractDescription.GetContract(typeof(ISecureService3)),
				binding,
				new EndpointAddress("https://localhost/wcf/example/security/test/service3"));
			ChannelFactory<ISecureService3> factory = 
				new ChannelFactory<ISecureService3>(ep);
			// 클라이언트 자격 증명(Windows 자격 증명) 설정
			factory.Credentials.Windows.ClientCredential =
				new NetworkCredential("Tester", "1234");
			ISecureService3 proxy1 = factory.CreateChannel();
			using (proxy1 as IDisposable) {
				proxy1.SecureMethod3("SSL with windows authentication service call");
				Console.WriteLine("SecureService3 (using SSL with windows authentication) invoked...");
			}
			// ClientBase<T>를 사용한 예제 코드
			using (SecureService3Client proxy2 = new SecureService3Client()) {
				proxy2.ClientCredentials.Windows.ClientCredential =
					new NetworkCredential("Tester", "1234");

				proxy2.SecureMethod3("SSL with windows authentication service call");
				Console.WriteLine("SecureService3 (using SSL with windows authentication) invoked...");
			}
		}

		// 디지털 서명만을 수행하는 메시지 보안 사용
		private static void InvokeService4()
		{
			// ChannelFactory<T>를 사용한 예제 코드
			BasicHttpBinding binding = new BasicHttpBinding();
			binding.Security.Mode = BasicHttpSecurityMode.Message;
			binding.Security.Message.ClientCredentialType = BasicHttpMessageCredentialType.Certificate;

			ServiceEndpoint ep = new ServiceEndpoint(
				ContractDescription.GetContract(typeof(ISecureService4)),
				binding,
				new EndpointAddress("http://localhost/wcf/example/security/test/service4"));
			ChannelFactory<ISecureService4> factory =
				new ChannelFactory<ISecureService4>(ep);
			// 서비스 인증서 설정
			factory.Credentials.ServiceCertificate.SetDefaultCertificate(
				"CN=localhost", StoreLocation.LocalMachine, StoreName.My);
			// 클라이언트 자격 증명(X.509 인증서 자격 증명) 설정
			factory.Credentials.ClientCertificate.SetCertificate(
				"CN=TestUser", StoreLocation.CurrentUser, StoreName.My);
			ISecureService4 proxy1 = factory.CreateChannel();
			using (proxy1 as IDisposable) {
				proxy1.SecureMethod4("Sign-only service call");
				Console.WriteLine("SecureService4 (sign only) invoked...");
			}
			// ClientBase<T>를 사용한 예제 코드
			using (SecureService4Client proxy2 = new SecureService4Client()) {
				// 서비스 인증서 설정
				proxy2.ClientCredentials.ServiceCertificate.SetDefaultCertificate(
					"CN=localhost", StoreLocation.LocalMachine, StoreName.My);
				// 클라이언트 인증서 설정
				proxy2.ClientCredentials.ClientCertificate.SetCertificate(
					"CN=TestUser", StoreLocation.CurrentUser, StoreName.My);

				proxy2.SecureMethod4("Sign-only service call");
				Console.WriteLine("SecureService4 (sign only) invoked...");
			}
		}

		// 디지털 서명과 암호화만을 수행하는 메시지 보안 사용
		private static void InvokeService5()
		{
			// ChannelFactory<T>를 사용한 예제 코드
			BasicHttpBinding binding = new BasicHttpBinding();
			binding.Security.Mode = BasicHttpSecurityMode.Message;
			binding.Security.Message.ClientCredentialType = BasicHttpMessageCredentialType.Certificate;

			ServiceEndpoint ep = new ServiceEndpoint(
				ContractDescription.GetContract(typeof(ISecureService5)),
				binding,
				new EndpointAddress("http://localhost/wcf/example/security/test/service5"));
			ChannelFactory<ISecureService5> factory =
				new ChannelFactory<ISecureService5>(ep);
			// 서비스 인증서 설정
			factory.Credentials.ServiceCertificate.SetDefaultCertificate(
				"CN=localhost", StoreLocation.LocalMachine, StoreName.My);
			// 클라이언트 자격 증명(X.509 인증서 자격 증명) 설정
			factory.Credentials.ClientCertificate.SetCertificate(
				"CN=TestUser", StoreLocation.CurrentUser, StoreName.My);
			ISecureService5 proxy1 = factory.CreateChannel();
			using (proxy1 as IDisposable) {
				proxy1.SecureMethod5("EncryptAndSign service call");
				Console.WriteLine("SecureService5 (encrypt and sign) invoked...");
			}
			// ClientBase<T>를 사용한 예제 코드
			using (SecureService5Client proxy2 = new SecureService5Client()) {
				// 서비스 인증서 설정
				proxy2.ClientCredentials.ServiceCertificate.SetDefaultCertificate(
					"CN=localhost", StoreLocation.LocalMachine, StoreName.My);
				// 클라이언트 인증서 설정
				proxy2.ClientCredentials.ClientCertificate.SetCertificate(
					"CN=TestUser", StoreLocation.CurrentUser, StoreName.My);

				proxy2.SecureMethod5("EncryptAndSign service call");
				Console.WriteLine("SecureService5 (encrypt and sign) invoked...");
			}
		}
	}
}
