﻿using System;
using System.Collections.Generic;
using System.ServiceModel;
using System.ServiceModel.Description;
using System.Runtime.Serialization;
using System.Reflection;

namespace DataContractInheritanceService
{
	[DataContract(Namespace = "http://noname.com/graphics/objects")]
	[KnownType(typeof(Rectangle))]
	[KnownType(typeof(Circle))]
	[KnownType(typeof(Arc))]
	//[KnownType("GetKnownTypes")]
	class Geometry
	{
		[DataMember]
		public int Id;
		[DataMember]
		public int Left = 0;
		[DataMember]
		public int Top = 0;
		[DataMember]
		public int Width = 0;
		[DataMember]
		public int Height = 0;

		public virtual void Draw()
		{
			Console.WriteLine("Drawing Geometry ({0},{1})-({2},{3})", Left, Top, Left + Width, Top + Height);
		}

		private static Type[] GetKnownTypes()
		{
			return new Type[] { typeof(Rectangle), typeof(Circle), typeof(Arc) };
		}
	}

	[DataContract(Namespace = "http://noname.com/graphics/objects")]
	class Rectangle : Geometry
	{
		[DataMember]
		public bool Fill = true;
		[DataMember]
		public int FillColor = 0;

		public override void Draw()
		{
			Console.Write("Drawing Rectangle ({0},{1})-({2},{3})", Left, Top, Left + Width, Top + Height);
			if (Fill) {
				Console.WriteLine(" Filling with color value {0}", FillColor);
			}
			else {
				Console.WriteLine(" Without fill");
			}
		}
	}

	[DataContract(Namespace = "http://noname.com/graphics/objects")]
	class Circle : Geometry
	{
		[DataMember]
		public bool Fill = true;
		[DataMember]
		public int Radius = 0;

		public override void Draw()
		{
			Console.WriteLine("Drawing Circle ({0},{1}) R={2}", Left, Top, Radius);
		}
	}

	[DataContract(Namespace = "http://noname.com/graphics/objects")]
	class Arc : Circle
	{
		[DataMember]
		public int StartAngle = 0;
		[DataMember]
		public int EndAngle = 90;

		public override void Draw()
		{
			Console.WriteLine("Drawing Rectangle ({0},{1}) R={2} Start={3}, End={4}", 
				Left, Top, Radius, StartAngle, EndAngle);
		}
	}

	// 서비스 계약 선언
	[ServiceContract(Namespace = "http://noname.com/graphics")]
	//[ServiceKnownType(typeof(Rectangle))]
	//[ServiceKnownType(typeof(Circle))]
	//[ServiceKnownType(typeof(Arc))]
	//[ServiceKnownType("GetKnownTypes", typeof(KnownTypeHelper))]
	interface IGraphics
	{
		[OperationContract]
		void AddGeometry(Geometry geometry);
		[OperationContract]
		Geometry GetGeometry(int id);
	}

	// Known type을 위한 Helper 클래스
	static class KnownTypeHelper
	{
		public static Type[] GetKnownTypes(ICustomAttributeProvider provider)
		{
			return new Type[] { typeof(Rectangle), typeof(Circle), typeof(Arc) };
		}
	}

	// 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class GraphicObjectService : IGraphics
	{
		private static Dictionary<int, Geometry> _Objects = new Dictionary<int, Geometry>();

		static GraphicObjectService()
		{
			// 테스트용 데이터 생성
			Geometry geometry = new Geometry();
			geometry.Id = 0;
			geometry.Left = geometry.Top = geometry.Width = geometry.Height = 0;
			_Objects.Add(geometry.Id, geometry);
			Circle circle = new Circle();
			circle.Id = 1;
			circle.Left = circle.Top = circle.Width = circle.Height = 100;
			circle.Fill = false;
			_Objects.Add(circle.Id, circle);
			Rectangle rectangle = new Rectangle();
			rectangle.Id = 2;
			rectangle.Left = rectangle.Top = rectangle.Width = rectangle.Height = 200;
			rectangle.Fill = true;
			rectangle.FillColor = 2;
			_Objects.Add(rectangle.Id, rectangle);
			Arc arc = new Arc();
			arc.Id = 3;
			arc.Left = arc.Top = arc.Width = arc.Height = 300;
			arc.StartAngle = 180;
			arc.EndAngle = 270;
			_Objects.Add(arc.Id, arc);
		}

		public void AddGeometry(Geometry geometry)
		{
			if (_Objects.ContainsKey(geometry.Id) == true) {
				_Objects[geometry.Id] = geometry;	// update
			}
			else {
				lock (_Objects) {
					_Objects.Add(geometry.Id, geometry);
					geometry.Draw();		// 가상 메쏘드 호출에 의해 적절한 클래스의 메쏘드가 호출된다.
				}
			}
		}

		public Geometry GetGeometry(int id)
		{
			if (_Objects.ContainsKey(id) == true) {
				return _Objects[id];
			}
			else {
				return null;
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Data Contract Inheritance Service Example...");

			ServiceHost host = new ServiceHost(
				typeof(GraphicObjectService),
				new Uri("http://localhost/wcf/example/testservice"));

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IGraphics),
				new BasicHttpBinding(),
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
