﻿using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Description;

namespace SerializationEventService
{
	[DataContract(Namespace = "http://noname.com/bookstore/data")]
	class BookInfo
	{
		[DataMember]
		public string Title;
		[DataMember(IsRequired = true)]
		public readonly string Isbn;
		[DataMember]
		public string Author;
		[DataMember]
		public int Price;
		// 이 필드는 클라이언트에게 알려지지 않는 필드이며
		// 직렬화에도 해당 사항이 없다.
		public double MarginRate;
		private int SavedPrice;

		public BookInfo(string title, string isbn, string author, int price) : this()
		{
			Title = title;
			Isbn = isbn;
			Author = author;
			Price = price;
			MarginRate = 0.25;
		}

		[OnSerializing]
		void BeforeSerialization(StreamingContext ctx)
		{
			SavedPrice = Price;
			Price = 10000;
			Console.WriteLine("OnSerializing event: before serialization, state={0}",
				ctx.State);
		}

		[OnSerialized]
		void AfterSerialization(StreamingContext ctx)
		{
			Price = SavedPrice;
			Console.WriteLine("OnSerialized event: after serialization, state={0}",
				ctx.State);
		}

		[OnDeserializing]
		void BeforeDeserialization(StreamingContext ctx)
		{
			MarginRate = 0.25;
			Console.WriteLine("OnDeserializing event: before deserialization, state={0}",
				ctx.State);
		}

		[OnDeserialized]
		void AfterDeserialization(StreamingContext ctx)
		{
			Console.WriteLine("OnDeserialized event: after deserialization, state={0}",
				ctx.State);
		}
	}

	// 서비스 계약 선언
	[ServiceContract(Namespace = "http://noname.com/bookstore")]
	interface IManageBooks
	{
		[OperationContract]
		void AddBook(BookInfo info);
		[OperationContract]
		BookInfo GetBookInfo(string isbn);
	}

	// 서비스 타입 구현
	[ServiceBehavior(IncludeExceptionDetailInFaults = true)]
	class BookStoreService : IManageBooks
	{
		private static Dictionary<string, BookInfo> _Books = new Dictionary<string, BookInfo>();

		static BookStoreService()
		{
			BookInfo info;

			info = new BookInfo("WCF 10일만에 정복", "000-001", "김정복", 40000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("10일동안 하면 개똥이 만큼 WCF 한다", "000-002", "김개똥", 42000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("WCF Programming", "000-003", "Mr. Programming", 53000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("Programming WCF", "000-004", "Mrs. Programming", 37000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("Learning WCF", "000-005", "Unknown", 18000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo("WCF Step by Step", "000-006", "김철수", 20000);
			_Books.Add(info.Isbn, info);
			info = new BookInfo(null, "000-007", null, 20000);
			_Books.Add(info.Isbn, info);
		}

		public void AddBook(BookInfo info)
		{
			if (_Books.ContainsKey(info.Isbn) == true) {
				_Books[info.Isbn] = info;	// update
			}
			else {
				lock (_Books) {
					_Books.Add(info.Isbn, info);
				}
				Console.WriteLine("MarineRate of new Book = {0:##0}%", info.MarginRate * 100);
			}
		}

		public BookInfo GetBookInfo(string isbn)
		{
			if (_Books.ContainsKey(isbn) == true) {
				return _Books[isbn];
			}
			else {
				return null;
			}
		}
	}

	class ServiceApp
	{
		static void Main(string[] args)
		{
			Console.WriteLine("Serialization Event Service Example...");

			ServiceHost host = new ServiceHost(
				typeof(BookStoreService),
				new Uri("http://localhost/wcf/example/bookstoreservice"));

			// 종점 추가
			host.AddServiceEndpoint(
				typeof(IManageBooks),
				new BasicHttpBinding(),
				"");

			// WSDL 을 위한 HTTP GET enable
			ServiceMetadataBehavior behavior = new ServiceMetadataBehavior();
			behavior.HttpGetEnabled = true;
			host.Description.Behaviors.Add(behavior);

			host.Open();
			Console.WriteLine("Press Any Key to Stop Service...");
			Console.ReadKey(true);
			host.Close();
		}
	}
}
